import java.util.*;
import java.io.*;

        /** Simple definition of a bank class.
         *  Allow PennyBankAccounts to be created and keep a list of them.
         *  A PennyBankAccount should only be created externally through
         *  the createAccount method.
         *  @author David J. Barnes (d.j.barnes@ukc.ac.uk)
         *  @version Version 2.1 (23rd Jun 2000)<br>
         *  @see PennyBankAccount
         */
public class SimpleBank {
    // An exception to indicate an unknown bank account.
    public static class UnknownAccountException extends RuntimeException {
        UnknownAccountException(long accountNumber){
            super("Unknown account with number: "+accountNumber);
        }
    }

    public SimpleBank(){
        gui.add(this);
    }

        // The means of creating a new account within this bank.
        // Each new account is given a unique account number.
    public PennyBankAccount createAccount(String accountName){
        // Give the account the next account number.
        setNumberOfAccounts(getNumberOfAccounts()+1);
        PennyBankAccount acc =
                new PennyBankAccount(this,accountName,getNumberOfAccounts());
        addAccount(acc);
        return acc;
    }

        // Return the account object with the given account number.
        // Throw an exception if there isn't one.
    public PennyBankAccount getAccount(long accountNumber)
                                throws UnknownAccountException {
        Long key = new Long(accountNumber);
        PennyBankAccount account = (PennyBankAccount) (getAccounts().get(key));
        if(account == null){
            throw new UnknownAccountException(accountNumber);
        }
        return account;
    }

        // List all known bank accounts.
    public void listAccounts(){
        final int headerWidth = 50;
        final PrintWriter writer = getWriter();
        writer.println(StringFormatter.center("Penny Bank Accounts",headerWidth));
        // Iterate over the accounts.
        Iterator accounts = getAccounts().values().iterator();
        while(accounts.hasNext()){
            PennyBankAccount account = (PennyBankAccount) accounts.next();
            writer.println(account);
        }
        writer.println("=====");
        writer.println();
        writer.flush();
    }

        // Add the account to the bank's collection. This 
        // should only be used internally.
    protected void addAccount(PennyBankAccount acc){
        if(acc != null){
            // Use the unique account number as the key.
            Long key = new Long(acc.getNumber());
            getAccounts().put(key,acc);
            // Tell the account which PrintWriter to use.
            acc.setWriter(getAccountWriter());
        }
    }

    protected HashMap getAccounts(){
        return accounts;
    }

    public long getNumberOfAccounts(){
        return numberOfAccounts;
    }

    protected void setNumberOfAccounts(long n){
        numberOfAccounts = n;
    }

        // The default may be changed to write to other than System.out.
    public void setWriter(PrintWriter w){
        writer = w;
    }

    protected PrintWriter getWriter(){
        return writer;
    }

        // The default may be changed to write to other than System.out.
    public void setAccountWriter(PrintWriter w){
        accountWriter = w;
    }

    protected PrintWriter getAccountWriter(){
        return accountWriter;
    }

    // Class variable used to assign new account numbers to each
    // account object. This is incremented before each new account
    // is created.
    private static long numberOfAccounts = 0;
    // Keep a record of all accounts.
    private final HashMap accounts = new HashMap();
    // Where bank reports are printed to. Standard output by default.
    private PrintWriter writer = new PrintWriter(System.out);
    // Where accounts report to. By default use same as back.
    private PrintWriter accountWriter = writer;
    // A GUI for the bank.
    private static final BankGUI gui = new BankGUI();
}
