        // A no-frills implementation of the VCR interface.
        // Basic operations and single speed FF and REW.
class NoFrillsVCR implements VCR {
    public boolean play(){
        if(getStatus() != NOTAPE){
            setStatus(PLAYING);
            getGui().start(PLAYING_SPEED);
            return true;
        }
        else{
            return false;
        }
    }
    
    public boolean pause(){
        if(getStatus() != NOTAPE){
            setStatus(PAUSED);
            // The GUI does not visually distinguish between stop and pause.
            getGui().stop();
            return true;
        }
        else{
            return false;
        }
    }
    
    public boolean stop(){
        if(getStatus() != NOTAPE){
            setStatus(STOPPED);
            getGui().stop();
            return true;
        }
        else{
            return false;
        }
    }
    
        // Only wind forward if the tape is stopped.
    public boolean ff(){
        if(getStatus() == STOPPED){
            setStatus(FORWARD);
            getGui().start(FAST_SPEED);
            return true;
        }
        else{
            return false;
        }
    }

        // Only wind backward if the tape is stopped.
    public boolean rew(){
        if(getStatus() == STOPPED){
            setStatus(BACKWARD);
            // Negative speed for reverse.
            getGui().start(-FAST_SPEED);
            return true;
        }
        else{
            return false;
        }
    }

        // Load the requested tape.
        // Return false if a tape is already loaded.
    public boolean loadTape(String tapeName){
        if(getStatus() == NOTAPE){
            setStatus(STOPPED);
            getGui().changeTape(tapeName);
            return true;
        }
        else{
            return false;
        }
    }

    public boolean eject(){
        if(getStatus() != NOTAPE){
            setStatus(NOTAPE);
            getGui().changeTape("");
            return true;
        }
        else{
            return false;
        }
    }

    public int getStatus(){
        return status;
    }

    private void setStatus(int s){
        status = s;
    }

    private TapeDemo getGui(){
        return gui;
    }

    // This object's current status attribute.
    private int status = NOTAPE;
    // The two speeds for normal and ff/rew.
    private static final int PLAYING_SPEED = 1, FAST_SPEED = 5;
    // A GUI to demonstrate this implementation.
    private TapeDemo gui = new TapeDemo("No Frills VCR",this);
    { gui.setLocation(20,20);
    }
}
