    // A generic server that accepts multiple client connections.
    // The service is implemented by the Class passed to the
    // constructor. The nature of this service is of no interest
    // to the listener.

import java.net.*;
import java.io.*;
import java.lang.reflect.Constructor;

class GenericListener {
    GenericListener(Class serviceType,int port) throws Exception {
        try{
            // The class of the slave service.
            this.serviceType = serviceType;
            // Listen on the given port.
            serverSocket = new ServerSocket(port);
        }
        catch(BindException e){
            throw new Exception("Failed to create a server socket: "+
                        e.getMessage());
        }
    }

        // Listen for clients. When each arrives, hand the socket
        // over to a slave.
    public void listen() throws Exception {
        ServerSocket serverSocket = getServerSocket();
        boolean keepWaiting = true;

        // Obtain a Constructor for the slave service so that
        // it can be instantiated.
        Class serviceType = getServiceType();
        // In order to be found, the constructor must be public.
        Constructor serviceConstructor =
                        serviceType.getConstructor(constructorArgs);
        // An array is needed for the constructor's actual parameter.
        // This is filled in with the socket reference once the accept
        // call returns.
        Object[] args = new Object[constructorArgs.length];

        while(keepWaiting){
            try{
                // Wait for a client to make contact.
                serverSocket.setSoTimeout(getTimeToWait());
                Socket clientSocket = serverSocket.accept();
                // Fill in the constructor's Socket parameter.
                args[0] = clientSocket;
                // Create an instance of the service class.
                Runnable slave = (Runnable) serviceConstructor.newInstance(args);
                // Let a separate Thread handle it.
                new Thread(slave).start();
            }
            catch(InterruptedIOException e){
                // We timed out waiting for a client.
                keepWaiting = false;
            }
            catch(IOException e){
                throw new Exception("IOException: "+ e.getMessage());
            }
        }
        // No more clients, so close the socket.
        serverSocket.close();
    }

    protected ServerSocket getServerSocket(){
        return serverSocket;
    }

    protected Class getServiceType(){
        return serviceType;
    }

    protected int getTimeToWait(){
        return timeToWait;
    }

    protected void setTimeToWait(int time){
        timeToWait = time;
    }

    // The class of the service offered by the slave.
    private final Class serviceType;
    // The constructor of the slave service is expected to
    // take a single parameter of class Socket.
    protected static final Class[] constructorArgs = {
        Socket.class,
    };
    // The socket on which the listening is done.
    private final ServerSocket serverSocket;
    // By default, wait up to 30 minutes for new clients.
    // After that, quit.
    private int timeToWait = 1*60*1000;
}
