        // A customer arrives to buy ice creams.
public class Arrival extends Event {
    public Arrival(long eventTime,Vendor v){
        super(eventTime);
        vendor = v;
    }

        // On arrival, schedule the next arrival and add the customer
        // to the vendor's queue.
    public void perform(Scheduler scheduler){
        long now = scheduler.getTime();

        Vendor vendor = getVendor();
        // Schedule the next arrival.
        scheduler.schedule(new Arrival(nextArrivalTime(now),vendor));

        // Inform the vendor.
        vendor.arrival(new Customer());
    }

    public String toString(){
        return getWhen()+" Arrival";
    }

        // Calculate when the next customer should arrive.
    protected long nextArrivalTime(long now){
        // Simulate a Poisson distribution of arrivals.
        return now + (long)(-meanArrivalTime*Math.log(Math.random()));
    }

    protected Vendor getVendor(){
        return vendor;
    }

    // From whom the ice creams are bought.
    private final Vendor vendor;
    // The mean time between customer arrivals.
    private final double meanArrivalTime = 60.0;
}
